angular.module('extensionDetector', ['ngAnimate', 'ui.bootstrap', 'rzModule', 'angularMoment'])
.config( [
    '$compileProvider', function( $compileProvider ) {   
        $compileProvider.aHrefSanitizationWhitelist(/^\s*(https?|chrome-extension|chrome):/); // to be able to show chrome:// and chrome-extension:// URLs
    } ]);
angular.module('extensionDetector').controller('ExtensionDetectorCtrl', function ($scope, $http) {
  $scope.agreed = false;
  $scope.lastupdate = 0; 
  $scope.agree = function() {
      $scope.agreed = true;
      $scope.reset();
  };

  $scope.maxWARslider = {
	  value: 1,
	  options: {
		  stepsArray: [1, 2, 3, 4, 5, 'unlimited'],
		showTicks: true,
		showTicksValues: true,
	  }
  };

  $scope.reset = function() {
      $scope.max = 0;
      $scope.offset = null;
      $scope.detectedExtensions = [];
      $scope.toTest = [];
      $scope.nextfile = null;
      $scope.warcount = 0;
      $scope.MaxWARPerExtension = 3;

      // detect browser
      $scope.inChrome = navigator.userAgent.indexOf("Chrome") >= 0;
      try {
	var x = chrome && chrome.webstore;
      } catch(e) {
	$scope.inChrome = false;
      }
      $scope.inFirefox = navigator.userAgent.indexOf("Firefox") >= 0 && !$scope.inChrome && (typeof InstallTrigger !== undefined);
      console.log("In chrome? "+$scope.inChrome);
      console.log("In firefox? "+$scope.inFirefox);

      // use different dataset per browser
      if($scope.inChrome) { $scope.nextfile = "data/chromedata0.json"; }
      if($scope.inFirefox) { $scope.nextfile = "data/firefoxdata0.json"; }

      $scope.fetchnextFile();
  }

  // log a detected extension and move on to the next
  $scope.logExtensionAndNext = function(u) {
      console.log("Detected extension with ID '"+$scope.currentExtension["id"]+"' via WAR '"+u+"'");
      $scope.currentExtension["detectedURI"] = u;
      $scope.detectedExtensions.push($scope.currentExtension);
      $scope.currentExtension = null;
      $scope.testNextExtension();
  }

  $scope.currentExtension = null;
  // test the next extension in the list
  $scope.testNextExtension = function() {
	  // currentExtension contains the extension we are checking for right now.
	  //  This extension may have several web accessible resources that all need to be checked.
	  //  As long as the extension is not detected and there are still web accessible resources to be checked,
	  //  currentExtension will not be null.
	  //  As soon as the extension has been detected, or there are no more WARs to check for, currentExtension becomes null.
	  //
	  // currentURI contains the next URI of the web accessible resource to check for next.
	  //  If this is null, there are no more WARs to check for this extension.
	  //
	  // toTest contains a list of all extensions to check for.
	  //  Only take the next element from this array if currentExtension is null.

	  var currentURI = null;

	  // if there is an extension under test right now, fetch the next WAR
	  // if there are no more WARs, forget this extension.
	  if($scope.currentExtension != null) {
	      if($scope.currentExtension["uris"].length > 0) {
		  // truncate WAR list if required
	          if($scope.maxWARslider.value != "unlimited" && $scope.currentExtension["uris"].length > $scope.maxWARslider.value) {
		      $scope.currentExtension["uris"] = $scope.currentExtension["uris"].slice(0, $scope.maxWARslider.value);
	          }
		  // get next WAR
		  currentURI = $scope.currentExtension["uris"][0];
		  $scope.currentExtension["uris"] = $scope.currentExtension["uris"].slice(1);
	      } else {
		  $scope.currentExtension = null;
	      }
	  }

	  // if there is no extension under test right now, get the next one from toTest and recurse
	  // if there are no more entries in toTest, fetch the next file
	  if($scope.currentExtension == null) {
	      if($scope.toTest.length > 0) {
		  $scope.currentExtension = $scope.toTest[0];
		  $scope.toTest = $scope.toTest.slice(1);
		  $scope.testNextExtension();
	      } else {
		  if($scope.nextfile) {
		      $scope.fetchnextFile();
		  } else {
		      $scope.offset = $scope.max;
		      if($scope.inFirefox) { $scope.$apply(); } // no clue why
		      console.log("Done.");
		  }
	      }
	  }

	  if(currentURI != null) {
	      $scope.warcount += 1;
	      if($scope.inChrome) {
		  $http.get(currentURI)
		     .then(function(res){
			  $scope.logExtensionAndNext(currentURI);
		      }, function(res) {
			  $scope.testNextExtension();
		      });
	      }
	      if($scope.inFirefox) {
		 tmp = angular.element('<script/>')
		    .on('load', function (event) { event.target.remove(); $scope.logExtensionAndNext(currentURI); })
		    .on('error', function (event) { event.target.remove(); $scope.testNextExtension(); });
		 tmp.attr("src", currentURI);
                 angular.element(document.body).append(tmp);
	      }
          }
  }
  // get the next datafile
  $scope.fetchnextFile = function() {
      if($scope.nextfile) {
	  $http.get($scope.nextfile)
	     .then(function(res){
		var data = res["data"];
		$scope.max = data['count'];
		$scope.offset = data['offset'];
		$scope.nextfile = data['nextfile'];
		$scope.toTest = data["data"];
		$scope.testNextExtension();
	      });
      }
  }

  $scope.updateTimestamp = function() {
      $http.get("timestamp.json")
	 .then(function(res){
	    var data = res["data"];
	    $scope.lastupdate = data["ts"] * 1000; // convert unix timestamp to JS timestamp
	  });
  };

  $scope.updateTimestamp();
});

function imgError(im) {
    im.style.display = 'none';
}


